*************************************************************************************
*														*
*	get string function for the EASy68k simulator	2010/08/23	V1.00			*
*														*
*	get_string returns a null terminated string which consists only of		*
*	characters from a set of valid characters or from all typable characters.	*
*	Only valid characters will be echoed to the screen or added to the returned	*
*	string.												*
*														*
*	The string is any length up to a pre defined maximum. This length includes	*
*	the null terminating character.								*
*														*
*	The user hits [ENTER] to return any entered string or [ESC] to return no	*
*	string. [BACKSPACE] can be used to edit the string as it is entered.		*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	 http://mycorner.no-ip.org/index.html							*
*														*
*	mail : leeedavison@googlemail.com								*
*														*
*************************************************************************************
*

	ORG	$1000

buff_len	EQU	16				* buffer is 16 characters including the
							* null terminator

start
	MOVEQ		#0,d1				* turn off echo
	MOVEQ		#12,d0			* keyboard echo
	TRAP		#15

	LEA		variables(pc),a3		* point to the variables

main
	LEA		prompt(pc),a1		* point to the prompt
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	LEA		valid(pc),a2		* point to the valid characters string
	LEA		buffer(a3),a1		* point to the buffer
	MOVEQ		#buff_len,d1		* set the maximum string length

	BSR		get_string			* get a string
	TST.w		d1				* test the response
	BGT.s		valid_string		* if the user didn't hit [ESC] or just [ENTER]
							* go display the returned string

	LEA		nullstring(pc),a1		* else point to the null string message
	BEQ.s		valid_string		* if the user just hit [ENTER] go display the
							* just hit enter message

	LEA		escaped(pc),a1		* else point to the escaped message
valid_string
	MOVEQ		#$0D,d1			* set [CR]
	MOVEQ		#6,d0				* display a single chracter
	TRAP		#15

	MOVEQ		#$0A,d1			* set [LF]
	TRAP		#15

	MOVEQ		#13,d0			* display a string with [CR][LF]
	TRAP		#15

	MOVEQ		#$0A,d1			* set [LF]
	MOVEQ		#6,d0				* display a single chracter
	TRAP		#15

	BRA.s		main				* loop forever

prompt
	dc.b	'Enter a number ?',$00		* the prompt

escaped
	dc.b	'The user hit [ESC]',$00	* the escape message

nullstring
	dc.b	'The user just hit [ENTER]',$00
							* the null string message

	ds.w	0					* ensure even


*************************************************************************************
*
* get a null terminated string made up from characters in the valid characters string
*
* [ENTER] returns the string
* [ESC] returns no string
* [BACKSPACE] can be used to edit the string
*
* a2 points to the null terminated valid characters string
* a1 points to the buffer
* d1 is the maximum string length including the null terminating byte
*
* if the valid character string has any character with b7 set then all keys will be
* allowed. this allows for no key checking with a single $FF byte at (a2)
*
* returns
* d1.w negative = user hit [ESC]
* d1.w positive = returned string length
*
* when the user hits [ESC] any entered string is deleted and the buffer terminated

get_string
	MOVEM.l	d0/d5-d7,-(sp)		* save the registers

	MOVE.w	d1,d7				* copy the maximum length
	SUBQ.w	#1,d7				* convert it to an index
	MOVEQ		#0,d6				* clear the index
get_key_loop
	MOVEQ		#7,d0				* set test key status
	TRAP		#15

	TST.b		d1				* is there a key waiting
	BNE.s		read_key			* if so go process it

* no key waiting so check for [ESC]

	MOVEQ		#$1B,d1			* set [], [], [], [ESC] key
	MOVEQ		#19,d0			* test keys
	TRAP		#15

	TST.b		d1				* has the escape key been pressed
	BEQ.s		get_key_loop		* if the [ESC] key is not pressed go check for
							* another key

* else wait for the [ESC] key to be released

get_esc
	MOVEQ		#$1B,d1			* set [], [], [], [ESC] key
	TRAP		#15

	TST.b		d1				* is the escape key pressed
	BNE.s		get_esc			* if the [ESC] key is pressed go wait some more

	MOVEQ		#-1,d1			* flag that the user hit [ESC]
	MOVEQ		#0,d6				* clear any string entered
	BRA.s		exit_get_string		* and exit

read_key
	MOVEQ		#5,d0				* get a single key
	TRAP		#15

	CMPI.b	#$0D,d1			* is it [CR]
	BEQ.s		hit_enter			* if it is [CR] go return the string

	CMPI.b	#$08,d1			* is it [BACKSPACE]
	BEQ.s		backspace			* if it is [BACKSPACE] go do it

	CMP.w		d6,d7				* compare the index with the maximum length
	BEQ.s		get_key_loop		* if the buffer is full just ignore the key

	MOVEQ		#-1,d5			* set the valid character string index
next_character
	ADDQ.w	#1,d5				* increment the valid character index
	MOVE.b	(a2,d5.w),d0		* get a valid character

	BMI.s		all_keys			* if the character is negative allow all keys

	BEQ.s		get_key_loop		* if end of the valid characters string just
							* ignore the key

	CMP.b		d0,d1				* compare the key with a valid character
	BNE.s		next_character		* if not valid go try the next character

all_keys
	MOVE.b	d1,(a1,d6.w)		* save the key
	BRA.s		show_key			* and go display it

backspace
	TST.w		d6				* test the buffer index
	BEQ.s		get_key_loop		* if the buffer is empty just ignore backspace

* else backup, overwrite the character with a [SPACE] then backup again. also decrement
* the buffer index

	MOVEQ		#6,d0				* display a single chracter
	TRAP		#15

	MOVEQ		#' ',d1			* set [SPACE]
	TRAP		#15

	SUBQ.w	#2,d6				* double decrement the index, once for the
							* [BACKSPACE] and once for the increment
	MOVEQ		#$08,d1			* set [BACKSPACE]
show_key
	MOVEQ		#6,d0				* display a single chracter
	TRAP		#15

	ADDQ.w	#1,d6				* increment the index
	BRA.s		get_key_loop		* go get another key

hit_enter
	MOVE.w	d6,d1				* return the string length
exit_get_string
	MOVEQ		#0,d0				* clear the character
	MOVE.b	d0,(a1,d6.w)		* null terminate the string

	MOVEM.l	(sp)+,d0/d5-d7		* restore the registers
	RTS


*************************************************************************************
*
* for this example the only valid keys are "0" to "9" and "."

valid
	dc.b	'0123456789.'			* decimal characters only
	dc.b	$00					* null terminator
	ds.w	0					* ensure even


*************************************************************************************
*
* this would normally be all the program variables but for this example is just some
* space for the input string buffer

variables
buffer					EQU	*-variables
	ds.b	buff_len				* input buffer space

	END		start


*************************************************************************************
